 #include <QtGui>
 #include <cmath>
 #include <iostream>
 #include "cartesiano.h"
 #include "mycartesiano.h"
using namespace std;

myCartesiano::myCartesiano(){
    
    //esegue paintEvent ogni 100 millisecondi
    QTimer *timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(update()));
    timer->start(10);

    //resetta il timer a 0
    time=new QTime;
    time->start();


    setWindowTitle(tr("Grafico"));
    resize(600, 600);
}
 

 void myCartesiano::paintEvent(QPaintEvent *)
 {
     QPainter painter(this);
 
     const double r=10.; // raggio del pendolo in metri
     const double thetamax=M_PI/4.;   //angolo di oscillazione del pendolo
     const double m=100,g=9.81; //massa del pengolo (in kg) e accelerazione di gravità g (in m/s^2)
    

     // dimensiona opportunamente le coordinate della finestra, 
     // in modo che le x e le y siano proporzionate 
     double rapp; 
     y1=0; 
     if(2*width()>height())
       rapp=(r*2)/(double)width();
     else
       rapp=r/(double)height();
     y0=-rapp*height();
     x0=-rapp*width()/2.;
     x1=-x0;

     // setta la scala del grafico x0,y0  
     painter.scale((width()) /fabs(x1-x0), -(height())/fabs(y1-y0));
     painter.translate(-x0, -y1);
       
     //variabili 
     static double xstart=-r*sin(thetamax);    //x di partenza
     static double ystart=-r*cos(thetamax);    //y di partenza
     static double theta=thetamax,xx=-xstart,yy=ystart,v=0;  //coordinate iniziali del pendolo (il perno e' situato in 0,0) e velocita' tangenziale iniziale
     static double periodo=0,a=0; 
     
     
     double f,t; 
     f=-g*m*sin(theta); //forza perpendicolare al filo del pendolo = risultante fra forza peso e tensione del filo 
     
     
     //tempo trascorso dall'ultima volta
     t=time->elapsed()/1000.;
     time->restart(); //fa ripartire il contatore
   
     
     //stima del periodo di oscillazione
     if(a*f<0){
	     if(xx<0.){
		     cout<<"Massa [kg]                 = "<<m<<endl; 
		     cout<<"Lunghezza [m]              = "<<r<<endl; 
		     cout<<"Accelerazione g [m/s^2]    = "<<g<<endl; 
		     cout<<"Angolo di oscillazione[deg]= "<<thetamax*360./2./M_PI<<endl; 
		     cout<<"periodo teorico [s]        = "<<2.*M_PI*sqrt(r/g) <<endl;   // corretta solo per piccoli angoli di oscillazione
		     cout<<"periodo \"sperimentale\" [s] = "<<periodo*2.<<endl;

                     //le approssimazioni sucessive del moto introducono degli errori: li stimiamo controllando l'errore sulla conservazione dell'energia 
		     double e0pot=-ystart*m*g;
		     double epot=-(ystart-yy)*m*g;
		     double ekin=0.5*v*v*m;  
		     cout <<"Errore sulla conservazione dell'energia Ekin+Epot rispetto all'energia potenziale iniziale: " << (ekin+epot)/e0pot*100.<<"% "<<endl;
	     }
	     periodo=0;
     }
     periodo=periodo+t;


     double sign=1; 
     if((v+a*t)+v<0) sign=-1;

     a=f/m; //accelerazione lungo la tangente del moto 
     theta=(a*t*t*0.5+t*v)/r+theta; //angolo 
     v=v+a*t;  //velocita' angolare
     xx=-r*sin(theta);  //x
     yy=-r*cos(theta);  //y
//     v=sqrt(2.*fabs(ystart-yy)*g)*sign; //calcola la velocità dalla conservazione dell'energia

     //disegna il segmento del pendolo
     QLineF line(xx,yy,0.,0.); 
     painter.drawLine(line);
      
     //disegna la pallina del pendolo
     QRectF c(xx-r/40.,yy-r/40,r/20,r/20);
     painter.drawEllipse(c);
     

 }
